@@ -49,6 +49,8 @@ extern int delete_mode;
 extern int cvs_exclude;
 extern int am_root;
 extern int relative_paths;
+extern int io_timeout;
+extern int io_error;
 
 /*
   free a sums struct
@@ -60,6 +62,75 @@ static void free_sums(struct sum_struct *s)
 }
 
 
+/*
+ * delete a file or directory. If force_delet is set then delete 
+ * recursively 
+ */
+static int delete_file(char *fname)
+{
+	DIR *d;
+	struct dirent *di;
+	char buf[MAXPATHLEN];
+	extern int force_delete;
+	struct stat st;
+	int ret;
+
+	if (do_unlink(fname) == 0 || errno == ENOENT) return 0;
+
+#if SUPPORT_LINKS
+	ret = lstat(fname, &st);
+#else
+	ret = stat(fname, &st);
+#endif
+	if (ret) {
+		fprintf(FERROR,"stat(%s) : %s\n", fname, strerror(errno));
+		return -1;
+	}
+
+	if (!S_ISDIR(st.st_mode)) {
+		fprintf(FERROR,"unlink(%s) : %s\n", fname, strerror(errno));
+		return -1;
+	}
+
+	if (do_rmdir(fname) == 0 || errno == ENOENT) return 0;
+	if (!force_delete || errno != ENOTEMPTY) {
+		fprintf(FERROR,"rmdir(%s) : %s\n", fname, strerror(errno));
+		return -1;
+	}
+
+	/* now we do a recsursive delete on the directory ... */
+	d = opendir(fname);
+	if (!d) {
+		fprintf(FERROR,"opendir(%s): %s\n",
+			fname,strerror(errno));
+		return -1;
+	}
+
+	for (di=readdir(d); di; di=readdir(d)) {
+		if (strcmp(di->d_name,".")==0 ||
+		    strcmp(di->d_name,"..")==0)
+			continue;
+		strncpy(buf, fname, (MAXPATHLEN-strlen(di->d_name))-2);
+		strcat(buf, "/");
+		strcat(buf, di->d_name);
+		buf[MAXPATHLEN-1] = 0;
+		if (verbose > 0)
+			fprintf(FINFO,"deleting %s\n", buf);
+		if (delete_file(buf) != 0) {
+			closedir(d);
+			return -1;
+		}
+	}	
+
+	closedir(d);
+	
+	if (do_rmdir(fname) != 0) {
+		fprintf(FERROR,"rmdir(%s) : %s\n", fname, strerror(errno));
+		return -1;
+	}
+
+	return 0;
+}
 
 /*
   send a sums struct down a fd
@@ -112,7 +183,7 @@ static struct sum_struct *generate_sums(struct map_struct *buf,off_t len,int n)
   }
 
   if (verbose > 3)
-    fprintf(FERROR,"count=%d rem=%d n=%d flength=%d\n",
+    fprintf(FINFO,"count=%d rem=%d n=%d flength=%d\n",
 	    s->count,s->remainder,s->n,(int)s->flength);
 
   s->sums = (struct sum_buf *)malloc(sizeof(s->sums[0])*s->count);
@@ -130,7 +201,7 @@ static struct sum_struct *generate_sums(struct map_struct *buf,off_t len,int n)
     s->sums[i].i = i;
 
     if (verbose > 3)
-      fprintf(FERROR,"chunk[%d] offset=%d len=%d sum1=%08x\n",
+      fprintf(FINFO,"chunk[%d] offset=%d len=%d sum1=%08x\n",
 	      i,(int)s->sums[i].offset,s->sums[i].len,s->sums[i].sum1);
 
     len -= n1;
@@ -159,7 +230,7 @@ static struct sum_struct *receive_sums(int f)
   s->sums = NULL;
 
   if (verbose > 3)
-    fprintf(FERROR,"count=%d n=%d rem=%d\n",
+    fprintf(FINFO,"count=%d n=%d rem=%d\n",
 	    s->count,s->n,s->remainder);
 
   if (s->count == 0) 
@@ -183,7 +254,7 @@ static struct sum_struct *receive_sums(int f)
     offset += s->sums[i].len;
 
     if (verbose > 3)
-      fprintf(FERROR,"chunk[%d] len=%d offset=%d sum1=%08x\n",
+      fprintf(FINFO,"chunk[%d] len=%d offset=%d sum1=%08x\n",
 	      i,s->sums[i].len,(int)s->sums[i].offset,s->sums[i].sum1);
   }
 
@@ -223,7 +294,7 @@ static int set_perms(char *fname,struct file_struct *file,struct stat *st,
   if (preserve_perms && !S_ISLNK(st->st_mode) &&
       st->st_mode != file->mode) {
     updated = 1;
-    if (chmod(fname,file->mode) != 0) {
+    if (do_chmod(fname,file->mode) != 0) {
       fprintf(FERROR,"failed to set permissions on %s : %s\n",
 	      fname,strerror(errno));
       return 0;
@@ -233,21 +304,24 @@ static int set_perms(char *fname,struct file_struct *file,struct stat *st,
 
   if ((am_root && preserve_uid && st->st_uid != file->uid) || 
       (preserve_gid && st->st_gid != file->gid)) {
-    updated = 1;
-    if (lchown(fname,
-	       (am_root&&preserve_uid)?file->uid:-1,
-	       preserve_gid?file->gid:-1) != 0) {
-      if (verbose>1 || preserve_uid)
-	fprintf(FERROR,"chown %s : %s\n",fname,strerror(errno));
-      return updated;
-    }
+	  if (do_lchown(fname,
+			(am_root&&preserve_uid)?file->uid:-1,
+			preserve_gid?file->gid:-1) != 0) {
+		  if (preserve_uid && st->st_uid != file->uid)
+			  updated = 1;
+		  if (verbose>1 || preserve_uid)
+			  fprintf(FERROR,"chown %s : %s\n",
+				  fname,strerror(errno));
+		  return updated;
+	  }
+	  updated = 1;
   }
     
   if (verbose > 1 && report) {
-    if (updated)
-      fprintf(FINFO,"%s\n",fname);
-    else
-      fprintf(FINFO,"%s is uptodate\n",fname);
+	  if (updated)
+		  fprintf(FINFO,"%s\n",fname);
+	  else
+		  fprintf(FINFO,"%s is uptodate\n",fname);
   }
   return updated;
 }
@@ -277,6 +351,16 @@ static int skip_file(char *fname,
 }
 
 
+/* use a larger block size for really big files */
+int adapt_block_size(struct file_struct *file, int bsize)
+{
+	int ret = file->length / (10000); /* rough heuristic */
+	ret = ret & ~15; /* multiple of 16 */
+	if (ret < bsize) ret = bsize;
+	if (ret > CHUNK_SIZE/2) ret = CHUNK_SIZE/2;
+	return ret;
+}
+
 void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 {  
   int fd;
@@ -284,17 +368,17 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
   struct map_struct *buf;
   struct sum_struct *s;
   int statret;
-  struct file_struct *file = &flist->files[i];
+  struct file_struct *file = flist->files[i];
 
   if (verbose > 2)
-    fprintf(FERROR,"recv_generator(%s,%d)\n",fname,i);
+    fprintf(FINFO,"recv_generator(%s,%d)\n",fname,i);
 
   statret = link_stat(fname,&st);
 
   if (S_ISDIR(file->mode)) {
     if (dry_run) return;
     if (statret == 0 && !S_ISDIR(st.st_mode)) {
-      if (unlink(fname) != 0) {
+      if (do_unlink(fname) != 0) {
 	fprintf(FERROR,"unlink %s : %s\n",fname,strerror(errno));
 	return;
       }
@@ -327,8 +411,8 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 	}
       }
     }
-    if (!dry_run) unlink(fname);
-    if (!dry_run && symlink(file->link,fname) != 0) {
+    delete_file(fname);
+    if (do_symlink(file->link,fname) != 0) {
       fprintf(FERROR,"link %s -> %s : %s\n",
 	      fname,file->link,strerror(errno));
     } else {
@@ -346,12 +430,11 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
     if (statret != 0 || 
 	st.st_mode != file->mode ||
 	st.st_rdev != file->rdev) {	
-      if (!dry_run) unlink(fname);
+      delete_file(fname);
       if (verbose > 2)
-	fprintf(FERROR,"mknod(%s,0%o,0x%x)\n",
+	fprintf(FINFO,"mknod(%s,0%o,0x%x)\n",
 		fname,(int)file->mode,(int)file->rdev);
-      if (!dry_run && 
-	  mknod(fname,file->mode,file->rdev) != 0) {
+      if (do_mknod(fname,file->mode,file->rdev) != 0) {
 	fprintf(FERROR,"mknod %s : %s\n",fname,strerror(errno));
       } else {
 	set_perms(fname,file,NULL,0);
@@ -367,12 +450,12 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 
   if (preserve_hard_links && check_hard_link(file)) {
     if (verbose > 1)
-      fprintf(FINFO,"%s is a hard link\n",file->name);
+      fprintf(FINFO,"%s is a hard link\n",f_name(file));
     return;
   }
 
   if (!S_ISREG(file->mode)) {
-    fprintf(FERROR,"skipping non-regular file %s\n",fname);
+    fprintf(FINFO,"skipping non-regular file %s\n",fname);
     return;
   }
 
@@ -388,15 +471,7 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
   }
 
   if (!S_ISREG(st.st_mode)) {
-    /* its not a regular file on the receiving end, but it is on the
-       sending end. If its a directory then skip it (too dangerous to
-       do a recursive deletion??) otherwise try to unlink it */
-    if (S_ISDIR(st.st_mode)) {
-      fprintf(FERROR,"ERROR: %s is a directory\n",fname);
-      return;
-    }
-    if (unlink(fname) != 0) {
-      fprintf(FERROR,"%s : not a regular file (generator)\n",fname);
+    if (delete_file(fname) != 0) {
       return;
     }
 
@@ -408,7 +483,7 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
 
   if (update_only && st.st_mtime > file->modtime) {
     if (verbose > 1)
-      fprintf(FERROR,"%s is newer\n",fname);
+      fprintf(FINFO,"%s is newer\n",fname);
     return;
   }
 
@@ -444,12 +519,12 @@ void recv_generator(char *fname,struct file_list *flist,int i,int f_out)
   }
 
   if (verbose > 3)
-    fprintf(FERROR,"gen mapped %s of size %d\n",fname,(int)st.st_size);
+    fprintf(FINFO,"gen mapped %s of size %d\n",fname,(int)st.st_size);
 
-  s = generate_sums(buf,st.st_size,block_size);
+  s = generate_sums(buf,st.st_size,adapt_block_size(file, block_size));
 
   if (verbose > 2)
-    fprintf(FERROR,"sending sums for %d\n",i);
+    fprintf(FINFO,"sending sums for %d\n",i);
 
   write_int(f_out,i);
   send_sums(s,f_out);
@@ -482,7 +557,7 @@ static int receive_data(int f_in,struct map_struct *buf,int fd,char *fname)
   for (i=recv_token(f_in,&data); i != 0; i=recv_token(f_in,&data)) {
     if (i > 0) {
       if (verbose > 3)
-	fprintf(FERROR,"data recv %d at %d\n",i,(int)offset);
+	fprintf(FINFO,"data recv %d at %d\n",i,(int)offset);
 
       sum_update(data,i);
 
@@ -499,7 +574,7 @@ static int receive_data(int f_in,struct map_struct *buf,int fd,char *fname)
 	len = remainder;
 
       if (verbose > 3)
-	fprintf(FERROR,"chunk[%d] of size %d at %d offset=%d\n",
+	fprintf(FINFO,"chunk[%d] of size %d at %d offset=%d\n",
 		i,len,(int)offset2,(int)offset);
 
       map = map_ptr(buf,offset2,len);
@@ -525,7 +600,7 @@ static int receive_data(int f_in,struct map_struct *buf,int fd,char *fname)
   if (remote_version >= 14) {
     read_buf(f_in,file_sum2,MD4_SUM_LENGTH);
     if (verbose > 2)
-      fprintf(FERROR,"got file_sum\n");
+      fprintf(FINFO,"got file_sum\n");
     if (fd != -1 && memcmp(file_sum1,file_sum2,MD4_SUM_LENGTH) != 0)
       return 0;
   }
@@ -536,17 +611,17 @@ static int receive_data(int f_in,struct map_struct *buf,int fd,char *fname)
 static void delete_one(struct file_struct *f)
 {
   if (!S_ISDIR(f->mode)) {
-    if (!dry_run && unlink(f->name) != 0) {
-      fprintf(FERROR,"unlink %s : %s\n",f->name,strerror(errno));
+    if (do_unlink(f_name(f)) != 0) {
+      fprintf(FERROR,"unlink %s : %s\n",f_name(f),strerror(errno));
     } else if (verbose) {
-      fprintf(FERROR,"deleting %s\n",f->name);
+      fprintf(FINFO,"deleting %s\n",f_name(f));
     }
   } else {    
-    if (!dry_run && rmdir(f->name) != 0) {
+    if (do_rmdir(f_name(f)) != 0) {
       if (errno != ENOTEMPTY)
-	fprintf(FERROR,"rmdir %s : %s\n",f->name,strerror(errno));
+	fprintf(FERROR,"rmdir %s : %s\n",f_name(f),strerror(errno));
     } else if (verbose) {
-      fprintf(FERROR,"deleting directory %s\n",f->name);      
+      fprintf(FINFO,"deleting directory %s\n",f_name(f));      
     }
   }
 }
@@ -567,7 +642,7 @@ static int delete_already_done(struct file_list *flist,int j)
 
 	if (j == 0) return 0;
 
-	name = strdup(flist->files[j].name);
+	name = strdup(f_name(flist->files[j]));
 
 	if (!name) {
 		fprintf(FERROR,"out of memory in delete_already_done");
@@ -583,7 +658,7 @@ static int delete_already_done(struct file_list *flist,int j)
 
 	while (low != high) {
 		int mid = (low+high)/2;
-		int ret = strcmp(flist->files[flist_up(flist, mid)].name,name);
+		int ret = strcmp(f_name(flist->files[flist_up(flist, mid)]),name);
 		if (ret == 0) {
 			free(name);
 			return 1;
@@ -597,7 +672,7 @@ static int delete_already_done(struct file_list *flist,int j)
 
 	low = flist_up(flist, low);
 
-	if (strcmp(flist->files[low].name,name) == 0) {
+	if (strcmp(f_name(flist->files[low]),name) == 0) {
 		free(name);
 		return 1;
 	}
@@ -618,10 +693,15 @@ static void delete_files(struct file_list *flist)
   if (cvs_exclude)
     add_cvs_excludes();
 
-  for (j=0;j<flist->count;j++) {
-	  char *name = flist->files[j].name;
+  if (io_error) {
+	  fprintf(FINFO,"IO error encountered - skipping file deletion\n");
+	  return;
+  }
 
-	  if (!S_ISDIR(flist->files[j].mode)) continue;
+  for (j=0;j<flist->count;j++) {
+	  char *name = f_name(flist->files[j]);
+
+	  if (!S_ISDIR(flist->files[j]->mode)) continue;
 
 	  if (delete_already_done(flist, j)) continue;
 
@@ -632,27 +712,24 @@ static void delete_files(struct file_list *flist)
 		  fprintf(FINFO,"deleting in %s\n", name);
 
 	  for (i=local_file_list->count-1;i>=0;i--) {
-		  if (!local_file_list->files[i].name) continue;
-		  if (-1 == flist_find(flist,&local_file_list->files[i])) {
-			  delete_one(&local_file_list->files[i]);
+		  if (!local_file_list->files[i]->basename) continue;
+		  if (-1 == flist_find(flist,local_file_list->files[i])) {
+			  delete_one(local_file_list->files[i]);
 		  }    
 	  }
+	  flist_free(local_file_list);
   }
 }
 
-static char *cleanup_fname = NULL;
+static char *cleanup_fname;
 
 void exit_cleanup(int code)
 {
 	if (cleanup_fname)
-		unlink(cleanup_fname);
+		do_unlink(cleanup_fname);
 	signal(SIGUSR1, SIG_IGN);
 	if (code) {
-#ifdef GETPGRP_VOID
-		kill(-getpgrp(), SIGUSR1);
-#else
-		kill(-getpgrp(getpid()), SIGUSR1);
-#endif
+		kill_all(SIGUSR1);
 	}
 	exit(code);
 }
@@ -676,7 +753,7 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
   int recv_ok;
 
   if (verbose > 2) {
-    fprintf(FERROR,"recv_files(%d) starting\n",flist->count);
+    fprintf(FINFO,"recv_files(%d) starting\n",flist->count);
   }
 
   if (recurse && delete_mode && !local_name && flist->count>0) {
@@ -691,7 +768,7 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
 	  phase++;
 	  csum_length = SUM_LENGTH;
 	  if (verbose > 2)
-	    fprintf(FERROR,"recv_files phase=%d\n",phase);
+	    fprintf(FINFO,"recv_files phase=%d\n",phase);
 	  write_int(f_gen,-1);
 	  write_flush(f_gen);
 	  continue;
@@ -699,8 +776,8 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
 	break;
       }
 
-      file = &flist->files[i];
-      fname = file->name;
+      file = flist->files[i];
+      fname = f_name(file);
 
       if (local_name)
 	fname = local_name;
@@ -712,7 +789,7 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
       }
 
       if (verbose > 2)
-	fprintf(FERROR,"recv_files(%s)\n",fname);
+	fprintf(FINFO,"recv_files(%s)\n",fname);
 
       /* open the file */  
       fd1 = open(fname,O_RDONLY);
@@ -734,7 +811,7 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
       if (fd1 != -1 && st.st_size > 0) {
 	buf = map_file(fd1,st.st_size);
 	if (verbose > 2)
-	  fprintf(FERROR,"recv mapped %s of size %d\n",fname,(int)st.st_size);
+	  fprintf(FINFO,"recv mapped %s of size %d\n",fname,(int)st.st_size);
       } else {
 	buf = NULL;
       }
@@ -742,6 +819,7 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
       /* open tmp file */
       if (strlen(fname) > (MAXPATHLEN-8)) {
 	fprintf(FERROR,"filename too long\n");
+	if (buf) unmap_file(buf);
 	close(fd1);
 	continue;
       }
@@ -763,10 +841,10 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
 	close(fd1);
 	continue;
       }
-      fd2 = open(fnametmp,O_WRONLY|O_CREAT|O_EXCL,file->mode);
+      fd2 = do_open(fnametmp,O_WRONLY|O_CREAT|O_EXCL,file->mode);
       if (fd2 == -1 && relative_paths && errno == ENOENT && 
 	  create_directory_path(fnametmp) == 0) {
-	      fd2 = open(fnametmp,O_WRONLY|O_CREAT|O_EXCL,file->mode);
+	      fd2 = do_open(fnametmp,O_WRONLY|O_CREAT|O_EXCL,file->mode);
       }
       if (fd2 == -1) {
 	fprintf(FERROR,"open %s : %s\n",fnametmp,strerror(errno));
@@ -784,14 +862,14 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
       /* recv file data */
       recv_ok = receive_data(f_in,buf,fd2,fname);
 
+      if (buf) unmap_file(buf);
       if (fd1 != -1) {
-	if (buf) unmap_file(buf);
 	close(fd1);
       }
       close(fd2);
 
       if (verbose > 2)
-	fprintf(FERROR,"renaming %s to %s\n",fnametmp,fname);
+	fprintf(FINFO,"renaming %s to %s\n",fnametmp,fname);
 
       if (make_backups) {
 	char fnamebak[MAXPATHLEN];
@@ -817,11 +895,11 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
 		      } else {
 			      set_perms(fname,file,NULL,0);
 		      }
-		      unlink(fnametmp);
+		      do_unlink(fnametmp);
 	      } else {
 		      fprintf(FERROR,"rename %s -> %s : %s\n",
 			      fnametmp,fname,strerror(errno));
-		      unlink(fnametmp);
+		      do_unlink(fnametmp);
 	      }
       } else {
 	      set_perms(fname,file,NULL,0);
@@ -831,11 +909,14 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
 
 
       if (!recv_ok) {
-	if (verbose > 1)
-	  fprintf(FERROR,"redoing %s(%d)\n",fname,i);
-        if (csum_length == SUM_LENGTH)
-	  fprintf(FERROR,"ERROR: file corruption in %s\n",fname);
-	write_int(f_gen,i);
+	      if (csum_length == SUM_LENGTH) {
+		      fprintf(FERROR,"ERROR: file corruption in %s. File changed during transfer?\n",
+			      fname);
+	      } else {
+		      if (verbose > 1)
+			      fprintf(FINFO,"redoing %s(%d)\n",fname,i);
+		      write_int(f_gen,i);
+	      }
       }
     }
 
@@ -845,142 +926,144 @@ int recv_files(int f_in,struct file_list *flist,char *local_name,int f_gen)
   /* now we need to fix any directory permissions that were 
      modified during the transfer */
   for (i = 0; i < flist->count; i++) {
-	  struct file_struct *file = &flist->files[i];
-	  if (!file->name || !S_ISDIR(file->mode)) continue;
-	  recv_generator(file->name,flist,i,-1);
+	  struct file_struct *file = flist->files[i];
+	  if (!file->basename || !S_ISDIR(file->mode)) continue;
+	  recv_generator(f_name(file),flist,i,-1);
   }
 
   if (verbose > 2)
-    fprintf(FERROR,"recv_files finished\n");
+    fprintf(FINFO,"recv_files finished\n");
   
   return 0;
 }
 
 
 
-off_t send_files(struct file_list *flist,int f_out,int f_in)
+void send_files(struct file_list *flist,int f_out,int f_in)
 { 
   int fd;
   struct sum_struct *s;
   struct map_struct *buf;
   struct stat st;
   char fname[MAXPATHLEN];  
-  off_t total=0;
   int i;
   struct file_struct *file;
   int phase = 0;
+  int offset=0;
 
   if (verbose > 2)
-    fprintf(FERROR,"send_files starting\n");
+    fprintf(FINFO,"send_files starting\n");
 
   setup_nonblocking(f_in,f_out);
 
-  while (1) 
-    {
-      i = read_int(f_in);
-      if (i == -1) {
-	if (phase==0 && remote_version >= 13) {
-	  phase++;
-	  csum_length = SUM_LENGTH;
-	  write_int(f_out,-1);
-	  write_flush(f_out);
+  while (1) {
+	  i = read_int(f_in);
+	  if (i == -1) {
+		  if (phase==0 && remote_version >= 13) {
+			  phase++;
+			  csum_length = SUM_LENGTH;
+			  write_int(f_out,-1);
+			  write_flush(f_out);
+			  if (verbose > 2)
+				  fprintf(FINFO,"send_files phase=%d\n",phase);
+			  continue;
+		  }
+		  break;
+	  }
+
+	  file = flist->files[i];
+
+	  fname[0] = 0;
+	  if (file->basedir) {
+		  strncpy(fname,file->basedir,MAXPATHLEN-1);
+		  fname[MAXPATHLEN-1] = 0;
+		  if (strlen(fname) == MAXPATHLEN-1) {
+			  io_error = 1;
+			  fprintf(FERROR, "send_files failed on long-named directory %s\n",
+				  fname);
+			  return;
+		  }
+		  strcat(fname,"/");
+		  offset = strlen(file->basedir)+1;
+	  }
+	  strncat(fname,f_name(file),MAXPATHLEN-strlen(fname));
+	  
+	  if (verbose > 2) 
+		  fprintf(FINFO,"send_files(%d,%s)\n",i,fname);
+	  
+	  if (dry_run) {	
+		  if (!am_server && verbose)
+			  printf("%s\n",fname);
+		  write_int(f_out,i);
+		  continue;
+	  }
+
+	  s = receive_sums(f_in);
+	  if (!s) {
+		  io_error = 1;
+		  fprintf(FERROR,"receive_sums failed\n");
+		  return;
+	  }
+	  
+	  fd = open(fname,O_RDONLY);
+	  if (fd == -1) {
+		  io_error = 1;
+		  fprintf(FERROR,"send_files failed to open %s: %s\n",
+			  fname,strerror(errno));
+		  free_sums(s);
+		  continue;
+	  }
+	  
+	  /* map the local file */
+	  if (fstat(fd,&st) != 0) {
+		  io_error = 1;
+		  fprintf(FERROR,"fstat failed : %s\n",strerror(errno));
+		  free_sums(s);
+		  close(fd);
+		  return;
+	  }
+	  
+	  if (st.st_size > 0) {
+		  buf = map_file(fd,st.st_size);
+	  } else {
+		  buf = NULL;
+	  }
+	  
 	  if (verbose > 2)
-	    fprintf(FERROR,"send_files phase=%d\n",phase);
-	  continue;
-	}
-	break;
-      }
-
-      file = &flist->files[i];
-
-      fname[0] = 0;
-      if (file->dir) {
-	strncpy(fname,file->dir,MAXPATHLEN-1);
-	fname[MAXPATHLEN-1] = 0;
-      if (strlen(fname) == MAXPATHLEN-1) {
-        fprintf(FERROR, "send_files failed on long-named directory %s\n",
-                fname);
-        return -1;
-      }
-	strcat(fname,"/");
-      }
-      strncat(fname,file->name,MAXPATHLEN-strlen(fname));
-
-      if (verbose > 2) 
-	fprintf(FERROR,"send_files(%d,%s)\n",i,fname);
-
-      if (dry_run) {	
-	if (!am_server && verbose)
-	  printf("%s\n",fname);
-	write_int(f_out,i);
-	continue;
-      }
-
-      s = receive_sums(f_in);
-      if (!s) {
-	fprintf(FERROR,"receive_sums failed\n");
-	return -1;
-      }
-
-      fd = open(fname,O_RDONLY);
-      if (fd == -1) {
-	fprintf(FERROR,"send_files failed to open %s: %s\n",
-		fname,strerror(errno));
-	continue;
-      }
-  
-      /* map the local file */
-      if (fstat(fd,&st) != 0) {
-	fprintf(FERROR,"fstat failed : %s\n",strerror(errno));
-	close(fd);
-	return -1;
-      }
-      
-      if (st.st_size > 0) {
-	buf = map_file(fd,st.st_size);
-      } else {
-	buf = NULL;
-      }
-
-      if (verbose > 2)
-	fprintf(FERROR,"send_files mapped %s of size %d\n",
-		fname,(int)st.st_size);
-
-      write_int(f_out,i);
-
-      write_int(f_out,s->count);
-      write_int(f_out,s->n);
-      write_int(f_out,s->remainder);
-
-      if (verbose > 2)
-	fprintf(FERROR,"calling match_sums %s\n",fname);
-
-      if (!am_server && verbose)
-	printf("%s\n",fname);
-      
-      match_sums(f_out,s,buf,st.st_size);
-      write_flush(f_out);
-      
-      if (buf) unmap_file(buf);
-      close(fd);
-
-      free_sums(s);
-
-      if (verbose > 2)
-	fprintf(FERROR,"sender finished %s\n",fname);
-
-      total += st.st_size;
-    }
+		  fprintf(FINFO,"send_files mapped %s of size %d\n",
+			  fname,(int)st.st_size);
+	  
+	  write_int(f_out,i);
+	  
+	  write_int(f_out,s->count);
+	  write_int(f_out,s->n);
+	  write_int(f_out,s->remainder);
+	  
+	  if (verbose > 2)
+		  fprintf(FINFO,"calling match_sums %s\n",fname);
+	  
+	  if (!am_server && verbose)
+		  printf("%s\n",fname+offset);
+	  
+	  match_sums(f_out,s,buf,st.st_size);
+	  write_flush(f_out);
+	  
+	  if (buf) unmap_file(buf);
+	  close(fd);
+	  
+	  free_sums(s);
+	  
+	  if (verbose > 2)
+		  fprintf(FINFO,"sender finished %s\n",fname);
+  }
 
   if (verbose > 2)
-    fprintf(FERROR,"send files finished\n");
+	  fprintf(FINFO,"send files finished\n");
 
   match_report();
 
   write_int(f_out,-1);
   write_flush(f_out);
-
-  return total;
 }
 
 
@@ -991,13 +1074,13 @@ void generate_files(int f,struct file_list *flist,char *local_name,int f_recv)
   int phase=0;
 
   if (verbose > 2)
-    fprintf(FERROR,"generator starting pid=%d count=%d\n",
+    fprintf(FINFO,"generator starting pid=%d count=%d\n",
 	    (int)getpid(),flist->count);
 
   for (i = 0; i < flist->count; i++) {
-    struct file_struct *file = &flist->files[i];
+    struct file_struct *file = flist->files[i];
     mode_t saved_mode = file->mode;
-    if (!file->name) continue;
+    if (!file->basename) continue;
 
     /* we need to ensure that any directories we create have writeable
        permissions initially so that we can create the files within
@@ -1006,7 +1089,7 @@ void generate_files(int f,struct file_list *flist,char *local_name,int f_recv)
       file->mode |= S_IWUSR; /* user write */
     }
 
-    recv_generator(local_name?local_name:file->name,
+    recv_generator(local_name?local_name:f_name(file),
 		   flist,i,f);
 
     file->mode = saved_mode;
@@ -1017,23 +1100,27 @@ void generate_files(int f,struct file_list *flist,char *local_name,int f_recv)
   ignore_times=1;
 
   if (verbose > 2)
-    fprintf(FERROR,"generate_files phase=%d\n",phase);
+    fprintf(FINFO,"generate_files phase=%d\n",phase);
 
   write_int(f,-1);
   write_flush(f);
 
+  /* we expect to just sit around now, so don't exit on a timeout. If we
+     really get a timeout then the other process should exit */
+  io_timeout = 0;
+
   if (remote_version >= 13) {
     /* in newer versions of the protocol the files can cycle through
        the system more than once to catch initial checksum errors */
     for (i=read_int(f_recv); i != -1; i=read_int(f_recv)) {
-      struct file_struct *file = &flist->files[i];
-      recv_generator(local_name?local_name:file->name,
+      struct file_struct *file = flist->files[i];
+      recv_generator(local_name?local_name:f_name(file),
 		     flist,i,f);    
     }
 
     phase++;
     if (verbose > 2)
-      fprintf(FERROR,"generate_files phase=%d\n",phase);
+      fprintf(FINFO,"generate_files phase=%d\n",phase);
 
     write_int(f,-1);
     write_flush(f);
@@ -1041,7 +1128,7 @@ void generate_files(int f,struct file_list *flist,char *local_name,int f_recv)
 
 
   if (verbose > 2)
-    fprintf(FERROR,"generator wrote %d\n",write_total());
+    fprintf(FINFO,"generator wrote %ld\n",(long)write_total());
 }
 
 
